@@ -39,6 +39,7 @@ enum {
 	/* cardos driver */
 	SC_CARD_TYPE_CARDOS_BASE = 1000,
 	SC_CARD_TYPE_CARDOS_GENERIC,
+	SC_CARD_TYPE_CARDOS_M4_01,
 	SC_CARD_TYPE_CARDOS_M4_2,
 	SC_CARD_TYPE_CARDOS_M4_3,
 
@@ -79,6 +80,8 @@ enum {
 	SC_CARD_TYPE_SETCOS_FINEID_V2,
 	SC_CARD_TYPE_SETCOS_NIDEL,
 	SC_CARD_TYPE_SETCOS_44 = 6100,
+	SC_CARD_TYPE_SETCOS_EID_V2_0,
+	SC_CARD_TYPE_SETCOS_EID_V2_1,
 
 	/* starcos driver */
 	SC_CARD_TYPE_STARCOS_BASE = 7000,
@@ -114,7 +117,20 @@ enum {
 
 	/* PIV-II type cards */
 	SC_CARD_TYPE_PIV_II_BASE = 14000,
-	SC_CARD_TYPE_PIV_II_GENERIC
+	SC_CARD_TYPE_PIV_II_GENERIC,
+	
+	/* Muscle cards */
+	SC_CARD_TYPE_MUSCLE_BASE = 15000,
+	SC_CARD_TYPE_MUSCLE_GENERIC,
+
+	/* ACOS5 driver */
+	SC_CARD_TYPE_ACOS5_BASE = 16000,
+	SC_CARD_TYPE_ACOS5_GENERIC,
+
+	/* Athena APCOS cards */
+	SC_CARD_TYPE_ASEPCOS_BASE = 17000,
+	SC_CARD_TYPE_ASEPCOS_GENERIC,
+	SC_CARD_TYPE_ASEPCOS_JAVA
 };
 
 #ifdef __cplusplus
