@@ -83,16 +83,14 @@ static int	do_cardos_extract_pubkey(sc_card_t *card, int nr, u8 tag,
 #define CARDOS_ALGO_EXT_RSA_SIG_PURE	0x8a
 #define CARDOS_ALGO_PIN			0x87
 
-static inline void
-tlv_init(struct tlv *tlv, u8 *base, size_t size)
+static void tlv_init(struct tlv *tlv, u8 *base, size_t size)
 {
 	tlv->base = base;
 	tlv->end = base + size;
 	tlv->current = tlv->next = base;
 }
 
-static inline void
-tlv_next(struct tlv *tlv, u8 tag)
+static void tlv_next(struct tlv *tlv, u8 tag)
 {
 	assert(tlv->next + 2 < tlv->end);
 	tlv->current = tlv->next;
@@ -100,8 +98,7 @@ tlv_next(struct tlv *tlv, u8 tag)
 	*(tlv->next++) = 0;
 }
 
-static inline void
-tlv_add(struct tlv *tlv, u8 val)
+static void tlv_add(struct tlv *tlv, u8 val)
 {
 	assert(tlv->next + 1 < tlv->end);
 	*(tlv->next++) = val;
@@ -385,7 +382,7 @@ cardos_store_pin(sc_profile_t *profile, sc_card_t *card,
 {
 	struct sc_cardctl_cardos_obj_info args;
 	unsigned char	buffer[256];
-	unsigned char	pinpadded[16];
+	unsigned char	pinpadded[256];
 	struct tlv	tlv;
 	unsigned int	attempts, minlen, maxlen;
 	int		r;
@@ -624,7 +621,8 @@ static int cardos_put_key(sc_profile_t *profile, sc_card_t *card,
 	if (pin_id < 0)
 		pin_id = 0;
 
-	if (key_info->modulus_length > 1024 && card->type == SC_CARD_TYPE_CARDOS_M4_2) {
+	if (key_info->modulus_length > 1024 && (card->type == SC_CARD_TYPE_CARDOS_M4_2 ||
+	    card->type == SC_CARD_TYPE_CARDOS_M4_3)) {
 		r = cardos_store_key_component(card, algorithm, key_id, pin_id, 0,
 			key->p.data, key->p.len, 0, 0);
 		if (r != SC_SUCCESS)
