@@ -1,7 +1,7 @@
 /*
  * Cryptoflex specific operation for PKCS #15 initialization
  *
- * Copyright (C) 2002  Juha Yrjl <juha.yrjola@iki.fi>
+ * Copyright (C) 2002  Juha Yrjölä <juha.yrjola@iki.fi>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -492,8 +492,7 @@ cflex_delete_dummy_chvs(sc_profile_t *profile, sc_card_t *card,
 /*
  * Create a pin file
  */
-static inline void
-put_pin(sc_profile_t *profile, unsigned char *buf,
+static void put_pin(sc_profile_t *profile, unsigned char *buf,
 		const u8 *pin, size_t len, int retry)
 {
 	if (len > 8)
@@ -604,8 +603,7 @@ cflex_create_empty_pin_file(sc_profile_t *profile, sc_card_t *card,
 /*
  * Get private and public key file
  */
-int
-cflex_get_keyfiles(sc_profile_t *profile, sc_card_t *card,
+static int cflex_get_keyfiles(sc_profile_t *profile, sc_card_t *card,
 			const sc_path_t *df_path,
 			sc_file_t **prkf, sc_file_t **pukf)
 {
