@@ -87,16 +87,14 @@ static int	incrypto34_extract_pubkey(sc_card_t *, int,
 #define INCRYPTO34_DECIPHER_RSA		INCRYPTO34_ALGO_RSA_PURE
 #define INCRYPTO34_ALGO_PIN			0x87
 
-static inline void
-tlv_init(struct tlv *tlv, u8 *base, size_t size)
+static void tlv_init(struct tlv *tlv, u8 *base, size_t size)
 {
 	tlv->base = base;
 	tlv->end = base + size;
 	tlv->current = tlv->next = base;
 }
 
-static inline void
-tlv_next(struct tlv *tlv, u8 tag)
+static void tlv_next(struct tlv *tlv, u8 tag)
 {
 	assert(tlv->next + 2 < tlv->end);
 	tlv->current = tlv->next;
@@ -104,8 +102,7 @@ tlv_next(struct tlv *tlv, u8 tag)
 	*(tlv->next++) = 0;
 }
 
-static inline void
-tlv_add(struct tlv *tlv, u8 val)
+static void tlv_add(struct tlv *tlv, u8 val)
 {
 	assert(tlv->next + 1 < tlv->end);
 	*(tlv->next++) = val;
